{   Delphi component TFrame                                          }
{   Copyright 24 March 1998 at 14:21  Mik Tran                       }

unit Frame3ds;

interface

uses WinTypes, WinProcs, Messages, SysUtils, Classes, Controls,
     Forms, Graphics, StdCtrls;

type
  Tattach = (Atnone,AtLeft,Atright,Attop,AtBottom);
  TBevelStyle = (Bsnone, bsLowered, bsRaised);
  TFrame = class(TGraphicControl{TCustomControl})
    private
        TheBitmap:Tbitmap;
        FAlignment : TAlignment;
        Factived : boolean;
        FBorder : Integer;
        procedure AutoInitialize;
        Procedure SetAlignment(value: Talignment);
        Procedure setInactivebitmap(value : Tbitmap);
        Procedure setactivebitmap(value : Tbitmap);
        Procedure setactive(value : boolean);
        Procedure SetAttach(value : Tattach);
        Procedure SetBorder(value : Integer);
        Procedure SetBorderStyle(value : TBevelStyle);
    protected
        procedure Click; override;
        procedure Paint; override;
        Procedure drawboxBut(rc: Trect;t:Tattach);
    public
        FInactiveBitmap, FactiveBitmap : Tbitmap;
        FAttach : Tattach;
        FBorderStyle : TBevelStyle;
        constructor Create(AOwner: TComponent); override;
        destructor destroy; override;
    published
      { Published properties of the component }
        Property Align;
        property OnClick;
        Property Caption;
        Property Font;
        Property Color;
        Property Alignment: Talignment read FAlignment write setAlignment;
        Property InActiveBitmap : Tbitmap read FInactiveBitmap write setInactivebitmap;
        Property ActiveBitmap : Tbitmap read FactiveBitmap write setactivebitmap;
        Property Actived : boolean read Factived write setactive;
        Property Attach : Tattach read Fattach write SetAttach;
        Property Border : integer read Fborder write Setborder;
        Property BorderStyle : TBevelStyle read FBorderStyle write SetBorderStyle;
  end;

procedure Register;

implementation
procedure Frame3D(Canvas: TCanvas; var Rect: TRect; TopColor, BottomColor: TColor;
  Width: Integer);

  procedure DoRect;
  var
    TopRight, BottomLeft: TPoint;
  begin
    with Canvas, Rect do
    begin
      TopRight.X := Right;
      TopRight.Y := Top;
      BottomLeft.X := Left;
      BottomLeft.Y := Bottom;
      Pen.Color := TopColor;
      PolyLine([BottomLeft, TopLeft, TopRight]);
      Pen.Color := BottomColor;
      Dec(BottomLeft.X);
      PolyLine([TopRight, BottomRight, BottomLeft]);
    end;
  end;

begin
  Canvas.Pen.Width := 1;
  Dec(Rect.Bottom); Dec(Rect.Right);
  while Width > 0 do
  begin
    Dec(Width);
    DoRect;
    InflateRect(Rect, -1, -1);
  end;
  Inc(Rect.Bottom); Inc(Rect.Right);
end;

procedure Register;
begin
 RegisterComponents('Custom', [TFrame]);
end;
Procedure TFrame.SetBorder(value : Integer);
begin
if value > 0 then
  Fborder := value;
Paint;
end;
Procedure TFrame.SetBorderStyle(value : TBevelStyle);
begin
FBorderStyle := Value;
paint;
end;

Procedure TFrame.setInactivebitmap(value : Tbitmap);
begin
 FInactivebitmap.assign(value);
end;

Procedure TFrame. setactive(value : boolean);
begin
 Factived := value;
 Paint;
end;

Procedure TFrame.SetAttach(value : TAttach);
begin
 Fattach := value;
 paint;
end;

Procedure TFrame.setactivebitmap(value : Tbitmap);
begin
 Factivebitmap.assign(value);
 Paint;
end;

Destructor TFrame.destroy;
begin
 FInactiveBitmap.free;
 FactiveBitmap .free;
 Inherited destroy;
end;

procedure TFrame.SetAlignment(value: Talignment);
begin
 falignment := value;
 paint;
end;

procedure TFrame.AutoInitialize;
begin
 FInactiveBitmap:= Tbitmap.create;
 FactiveBitmap := Tbitmap.create;
 width := 60;
 height := 22;
 Font.name := 'Arial';
 Falignment := taCenter;
 Fattach := ATnone;
 FBorderStyle := bsRaised;
 FBorder :=1;
end; 

procedure TFrame.Click;
begin
     if not Factived then Actived:= true;
     inherited Click;
end;

constructor TFrame.Create(AOwner: TComponent);
begin
     inherited Create(AOwner);
     AutoInitialize;
end;

procedure fillcanvas(canvas: tcanvas; Bmp : Tbitmap;W,H: integer);
var ro,co : integer;
begin
if not bmp.empty then
FOR Ro := 0 TO H DIV bmp.Height DO
  FOR Co := 0 TO W DIV bmp.Width DO
    canvas.draw(co* bmp.Width, ro * bmp.Height, bmp);
end;

Procedure TFrame.drawboxBut(rc: Trect;t:Tattach);
  var
    TopRight, BottomLeft: TPoint;
    Topcolor, Botcolor: Tcolor;
  begin
  if FBorderStyle = bsRaised then
   begin
     topcolor := clBtnHighlight;
     Botcolor := clBtnShadow;
   end
  else
   Begin
     topcolor := clBtnShadow;
     Botcolor := clBtnHighlight;
   end;

  Canvas.Pen.Width := Fborder;
  Dec(Rc.Bottom); Dec(Rc.Right);
    with Canvas, Rc do
    begin
      TopRight.X := Right;
      TopRight.Y := Top;
      BottomLeft.X := Left;
      BottomLeft.Y := Bottom;
      Pen.Color := Topcolor;
      case t of
      atleft : begin
                   PolyLine([TopLeft, TopRight]);
                   Pen.Color := Botcolor;
                   Dec(BottomLeft.X);
                   PolyLine([TopRight, BottomRight, BottomLeft]);
               end;
      Attop :  begin
                   PolyLine([BottomLeft, TopLeft]);
                   Pen.Color := Botcolor;
                   Dec(BottomLeft.X);
                   PolyLine([TopRight, BottomRight, BottomLeft]);
              end;
      Atright: begin
                   PolyLine([BottomLeft, TopLeft, TopRight]);
                   Pen.Color := Botcolor;
                   Dec(BottomLeft.X);
                   PolyLine([BottomRight, BottomLeft]);
              end;
      AtBottom :begin
                   PolyLine([BottomLeft, TopLeft, TopRight]);
                   Pen.Color := Botcolor;
                   Dec(BottomLeft.X);
                   PolyLine([TopRight, BottomRight]);
              end;
      end;
    InflateRect(Rc, -1, -1);
    Inc(Rc.Bottom); Inc(Rc.Right);
    end;

end;


procedure TFrame.Paint;
var
  Rect: TRect;
  TopColor, BottomColor: TColor;
  Text: array[0..255] of Char;
  FontHeight: Integer;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);

begin
{  Factived := Focused;}
  Rect := GetClientRect;
  if factivebitmap.empty then
   begin
    canvas.brush.color := self.color;
    canvas.Brush.Style := bssolid;
    canvas.fillrect(Rect);
   end
  else
   begin
    If Factived then
     begin
      if (Factivebitmap.width >= width) and (Factivebitmap.height >= height) then
         canvas.draw(0,0,Factivebitmap)
      else
        fillcanvas(canvas,Factivebitmap,width,height);
     end
    Else
     begin
      if not fInactivebitmap.empty then
      if (FInactivebitmap.width >= width) and (FInactivebitmap.height >= height) then
         canvas.draw(0,0,FInactivebitmap)
      else
        fillcanvas(canvas,FInactivebitmap,width,height);
     end;
   end;
if Fborderstyle <> Bsnone then
case Fattach of
 Atnone:  if FBorderStyle = bsRaised then
            FRAME3D(Canvas,rect, clBtnHighlight,clBtnShadow,Fborder)
          else
            FRAME3D(Canvas,rect, clBtnShadow, clBtnHighlight,Fborder)
 else DrawboxBut(rect,Fattach);
end;
  with Canvas do
  begin
   Brush.Style := bsClear;
   Font := Self.Font;
   FontHeight := TextHeight('W');
   with Rect do
    begin
      Top := ((Bottom + Top) - FontHeight) shr 1;
      Bottom := Top + FontHeight;
    end;
    StrPCopy(Text, Caption);
    DrawText(Handle, Text, StrLen(Text), Rect, (DT_EXPANDTABS or
      DT_VCENTER) or Alignments[FAlignment]);
   end;
end;


end.
